import { ClubAuthId, UserAuthId } from '@/router/permissions';
import { z } from 'zod';

const createRespSchema = <T extends z.ZodTypeAny>(data: T) =>
  z.union([
    z
      .object({
        status: z.number().min(200).max(299),
        msg: z.string(),
        timestamp: z.coerce.date(),
      })
      .extend({ data: data })
      .transform((raw) =>
        Object.assign(raw, {
          type: 'success' as const,
        }),
      ),
    z
      .object({
        status: z.number(),
        msg: z.string(),
        timestamp: z.coerce.date(),
      })
      .transform((raw) =>
        Object.assign(raw, {
          type: 'error' as const,
        }),
      ),
  ]);
export type AnyRespSchema = ReturnType<typeof createRespSchema>;
export type SucceedRespOf<T extends z.infer<AnyRespSchema>> = Extract<T, { type: 'success' }>;
export type AnySucceedResp = SucceedRespOf<z.infer<AnyRespSchema>>;
export type ErrorResp = Extract<z.infer<AnyRespSchema>, { type: 'error' }>;
export const createPagedRespSchema = <T extends z.ZodTypeAny>(data: T) =>
  createRespSchema(
    z.object({
      data: data.array(),
      total: z.number(),
    }),
  );
export type AnyPagedRespSchema = ReturnType<typeof createPagedRespSchema>;
export type SucceedPagedDataOf<T extends z.infer<AnyPagedRespSchema>> = SucceedRespOf<T>['data'];
export type AnySucceedPagedResp = SucceedRespOf<z.infer<AnyPagedRespSchema>>;
export const ordinarySchema = createRespSchema(z.literal(true));
const createIdAndNameSchema = <T extends z.ZodTypeAny = z.ZodNumber>(
  id: T = z.number() as unknown as T,
) => z.object({ id, name: z.string() });
export const idAndNameSchema = createIdAndNameSchema();
const createAuthSchema = <T extends z.ZodTypeAny = z.ZodNumber>(id: T) =>
  createIdAndNameSchema(id).extend({
    permissions: idAndNameSchema.array(),
  });
const userAuthSchema = createAuthSchema(z.nativeEnum(UserAuthId));
const clubAuthSchema = createAuthSchema(z.nativeEnum(ClubAuthId));
export const clubSchema = idAndNameSchema.extend({
  avatar: z.optional(z.string()),
  commit: z.string(),
});
export type Club = z.infer<typeof clubSchema>;
export const clubRespSchema = createRespSchema(clubSchema);
export const clubListRespSchema = createPagedRespSchema(clubSchema);
const userInfoSchema = idAndNameSchema.extend({
  avatar: z.optional(z.string()),
  auth: userAuthSchema,
  club: z.optional(clubSchema),
  clubAuth: z.optional(clubAuthSchema),
});
export const userInfoRespSchema = createRespSchema(userInfoSchema);
export type UserInfo = SucceedRespOf<z.infer<typeof userInfoRespSchema>>['data'];
export const userInfoListRespSchema = createPagedRespSchema(userInfoSchema);
export const loginRespSchema = userInfoRespSchema;
export const registerRespSchema = userInfoRespSchema;
export const verifyRespSchema = createRespSchema(
  z
    .object({
      img: z.string(),
      key: z.string(),
    })
    .transform((raw) => {
      raw.img = `data:image/png;base64,${raw.img}`;
      return raw;
    }),
);
export const uploadAvatarRespSchema = createRespSchema(z.string());
